using System;
using System.Reflection;
using System.Drawing;
using System.IO;
using Microsoft.ApplicationBlocks.ExceptionManagement;

namespace Photo_Editor_Application
{
    //public delegate void EventHandler(object sender, EventArgs args);
    /// <summary>
    /// Summary description for PlugInInterface.
    /// </summary>
    public class PlugInInterface
    {
        // postfix for all dynamically loadable components
        private static string searchPlugInString = "PlugIn.dll";
        Assembly assembly;
        private Bitmap tmpBitmap;
        // values of the parameters
        private int param3;
        private int param2;
        private int param1;
        // name of the operation and prefix of the component
        private string sliderName;

        /// <summary>
        /// Constructor 
        /// </summary>
        public PlugInInterface()
        {
            //
            // TODO: Add constructor logic here
            //
            
        }
        /// <summary>
        /// Searches the PlugIns directory for a dll with the
        /// specified name. If none is found then an exception is thrown.
        /// Otherwise it tries to invoke the method ProcessImage with the specified
        /// parameters.
        /// </summary>
        public void ApplyImageProcessing()
        {
            try
            {
               
                #region ApplyImageProcessing Code from Chapter 8	
                // Concatenates the operation name with
                // the suffix "PlugIn.dll"
                string operation;
                operation = sliderName + searchPlugInString;

                string[] filenames = Directory.GetFiles(@".\", operation);

                // If no PlugIn is found indicate an error by 
                // throwing an exception.
                if(filenames.Length == 0)
                {
                    throw(new Exception("Error, PlugIn could not be found in the specified location"));
                }

                // This should never ever happen! Two files can not have the same name
                // and reside in the same directory!
                // But anyway the check does not hurt either. Better safe than sorry ;-)
                if(filenames.Length > 1)
                    throw(new Exception("Error, More than one PlugIn could be found in the specified location"));
		
                // Load the Plug In assembly that was found.
                assembly = Assembly.LoadFrom(filenames[0]);
			
                // Query the loaded assembly for types defined.
                Type[] theType = assembly.GetTypes();
		
                // Create an object arraay which holds the parameters for the 
                // method to be called.
                object[] param = new Object[4];
				
                param[0] = tmpBitmap;	
                param[1] = param1;
                param[2] = param2;
                param[3] = param3;
					
                // Get all the Methods defined in the PlugIn assembly
                MethodInfo[] methods = theType[0].GetMethods();
				
                // used to remember is a Method with name
                // methodName was found. If not an error will
                // be signaled.
                bool methodNotFound = true;

                // Defines the name of the method 
                // that needs to be defined in each PlugIn
                // in order to invoke it.
                const string methodName = "ProcessImage";
				
                object assemblyObject;
                // Iterate through the Methods defined
                // in the assembly that was found and
                // check if a method with the specified
                // Name is defined in the assembly.
                // If so Invoke the Method. 
                foreach(MethodInfo NextMethod in methods)
                {
                    #endregion
                    if(NextMethod.Name == methodName)
                    {
                        // Create an Instance of the type found in the PlugIn Assembly
                        assemblyObject = Activator.CreateInstance(theType[0]);
                        try
                        {
                            methodNotFound = false;
					
                            // Invoke the member which signature is
                            // void ProcessImage(Bitmap, int, int, int);
                            theType[0].InvokeMember(methodName,
                                BindingFlags.InvokeMethod,
                                null,
                                assemblyObject,
                                param);
                        } 
                            // Exception at his point most likely means that
                            // none of the mehtods with the correct Name
                            // had the correct signature!
                        catch(Exception e)
                        {
                            throw(new Exception(e.Message + " \n\n Method with the specified signature could NOT be found in PlugIn dll. Or an exception was thrown within the invoked method!"));
                        }
                    }
					
                }
                if(methodNotFound)
                    // If no method with the correct name was defined by the assembly 
                    // then indicate the error to the user!
                    throw( new Exception("Error PlugIn does NOT define a method with the name ProcessInmage!"));
                
            }
            catch(Exception exception)
            {
                // In case the Invokation is failing an
                // error message is displayed.
                ExceptionManager.Publish(exception);
            }
        }

        #region Accessor Methods and Property declarations
        /// <summary>
        /// Accessor for SliderName which represents the
        /// PlugIn to be used for the calculation requested.
        /// </summary>
        public string SliderName
        {
            get
            {
                return sliderName;
            }
            set
            {
                sliderName = value;
            }
        }
		
        /// <summary>
        /// Accessor for param to set the slider values.
        /// </summary>
        public int Param1
        {
            get
            {
                return param1;
            }
            set
            {
                param1 = value;
            }
        }
		
        /// <summary>
        /// Accessor for param to set the slider values.
        /// </summary>
        public int Param2
        {
            get
            {
                return param2;
            }
            set
            {
                param2 = value;
            }
        }
		
        /// <summary>
        /// Accessor for param to set the slider values.
        /// </summary>
        public int Param3
        {
            get
            {
                return param3;
            }
            set
            {
                param3 = value;
            }
        }

        /// <summary>
        /// Accessor for the image on which
        /// the calculation is performed.
        /// </summary>
        public Bitmap TmpBitmap
        {
            set
            {
                tmpBitmap = value;
            }
            get
            {
                return tmpBitmap;
            }
        }
        
        #endregion
    }
}
