using System;
using System.Drawing;

namespace Photo_Editor_Application
{
    /// <summary>
    /// The RectangleComponent class implements the
    /// functionality to draw a rectangle into the picture.
    /// </summary>
    /// <requirement>F:image_graphics_annotations</requirement>
    public class RectangleComponent : GraphicsComponent
    {
        /// <summary>
        /// Constructs a Rectangle object.
        /// </summary>
        public RectangleComponent()
        {

        }

        /// <summary>
        /// Method that draws the rectangle component into the 
        /// graphics
        /// object. The DrawRectangle method provided by GDI+ is 
        /// used to
        /// actually draw the graphic primitive. Depending on the /// mouse
        /// movement the width height and position are adjusted.
        /// </summary>
        public override void Draw()
        {
            int width =  CalcViewportCoords(EndPoint).X - CalcViewportCoords(StartPoint).X;
            int height = CalcViewportCoords(EndPoint).Y - CalcViewportCoords(StartPoint).Y;

            // if width and height are less than zero than 
            //exchange start and end point.
            if(width < 0 && height < 0)
                DeviceContext.DrawRectangle(DrawingPen, 
                    CalcViewportCoords(EndPoint).X, 
                    CalcViewportCoords(EndPoint).Y, 
                    Math.Abs(width), Math.Abs(height));
            else if (width < 0)
                // if just width < 0 than x coordinate of start 
                // and end point have to be 
                // swapped in order to draw the rectangle with 
                // positive width and height.
                DeviceContext.DrawRectangle(DrawingPen, 
                    CalcViewportCoords(EndPoint).X, 
                    CalcViewportCoords(StartPoint).Y, 
                    Math.Abs(width), height);
            else if (height < 0)
                // if just height < 0 than y coordinate of start 
                // and end point have to be 
                // swapped in order to draw the rectangle with 
                // positive width and height.
                DeviceContext.DrawRectangle(DrawingPen, 
                    CalcViewportCoords(StartPoint).X, 
                    CalcViewportCoords(EndPoint).Y, 
                    width, Math.Abs(height));
            else 
                // if everything is OK then just draw the 
                // rectangle using the provided coordinates.
                DeviceContext.DrawRectangle(DrawingPen, 
                    CalcViewportCoords(StartPoint).X, 
                    CalcViewportCoords(StartPoint).Y, 
                    width, height);
        }
    }
}
